package be.iminds.ilabt.jfed.preferences;

import be.iminds.ilabt.jfed.git.GitAuthPreferences;
import be.iminds.ilabt.jfed.lowlevel.user.GeniUser;
import be.iminds.ilabt.jfed.lowlevel.lib.RetrySettings;
import be.iminds.ilabt.jfed.lowlevel.ssh_key_info.PuTTYFilesKeyInfo;
import be.iminds.ilabt.jfed.util.common.OSDetector;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.io.File;
import java.util.List;

/**
 * JFedPreferences
 */
public interface JFedPreferences {
    /**
     * Increment nr to force new "FirstRun" for new version.
     * Default nr is 0. Will be set to this nr on firstrun handling.
     */
    int CURRENT_FIRSTRUN_VERSION = 1;

    void fillinBlanks();

    boolean containsPreference(@Nonnull PreferenceKey key);

    boolean hasKey(@Nonnull PreferenceKey key);

    @Nullable
    String getString(@Nonnull PreferenceKey key);
    @Nonnull
    List<String> getStringList(@Nonnull PreferenceKey key);

    void setString(@Nonnull PreferenceKey key, @Nonnull String value);
    void setStringList(@Nonnull PreferenceKey key, @Nonnull List<String> value);

    @Nullable Integer getInt(@Nonnull PreferenceKey key);

    int getInt(@Nonnull PreferenceKey key, int defaultValue);

    void setInt(@Nonnull PreferenceKey key, int value);

    @Nullable Boolean getBoolean(@Nonnull PreferenceKey key);

    boolean getBoolean(@Nonnull PreferenceKey key, boolean defaultValue);

    void setBoolean(@Nonnull PreferenceKey key, boolean value);

    @Nullable File getFile(@Nonnull PreferenceKey key);

    void setFile(@Nonnull PreferenceKey key, @Nonnull File value);

    void clear(@Nonnull PreferenceKey key);

    @Nullable File getPropertiesFile();

    void addPreferenceChangeListener(@Nonnull PreferenceKey prefDebugCpu, @Nonnull PreferenceChangeListener preferenceChangeListener);


    void setRetrySettings(@Nonnull RetrySettings retrySettings);

    @Nonnull RetrySettings getRetrySettings();

    @Nullable GitAuthPreferences getGitAuthPreferences(GeniUser user);

    @Nullable PuTTYFilesKeyInfo getCustomSshKeyInfo();

    /**
     * Preference
     */
    interface PreferenceKey {
        @Override
        String toString();

        @Nonnull
        String getKey();

        boolean isOSSpecific();

        @Nullable
        OSDetector.OSFilter getOSFilter();
    }
}
